#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Command-line interface to kick off the crawl process."""

import argparse
from parker.workers import crawler

parser = argparse.ArgumentParser(
    description=(
        'Set-up a URI to crawl with the '
        'relevant site configuration.'
    )
)
parser.add_argument(
    'site',
    type=str,
    help='the name of the site (used to load the site config)'
)
parser.add_argument(
    '--uri',
    type=str,
    help='the URI we wish to crawl'
)

args = parser.parse_args()

crawler(args.site, args.uri)



# -*- coding: utf-8 -*-
"""HTTP client for Parker."""

import random
import requests
from configloader import load_config

DEFAULT_UA = "Parker v0.1.0"
_PERMITTED_STATUS_CODES = [200]
_instances = dict()


def get_instance():
    """Return an instance of Client."""
    config = load_config('client')
    user_agents = config['user-agents']
    proxies = config['proxies']

    user_agent = user_agents[
        random.randint(0, len(user_agents) - 1)
    ] if len(user_agents) > 0 else DEFAULT_UA

    proxy = proxies[
        random.randint(0, len(proxies) - 1)
    ] if len(proxies) > 0 else None

    try:
        instance = _instances[user_agent]
    except KeyError:
        instance = Client(user_agent, proxy)
        _instances[user_agent] = instance

    return instance


class Client(object):

    """HTTP client object."""

    headers = {
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,\
            image/webp,*/*;q=0.8',
        'accept-encoding': 'gzip,deflate,sdch',
        'accept-language': 'en-GB,en-US;q=0.8,en;q=0.6',
        'cache-control': 'no-cache',
        'connection': 'keep-alive',
        'pragma': 'no-cache'
    }

    def __init__(self, user_agent, proxy=False):
        """Constructor."""
        self.headers["user_agent"] = user_agent
        self.proxy = proxy
        self.response_headers = None

    def __repr__(self):
        """Return an unambiguous representation."""
        return "%s(%s)" % (self.__class__, self.headers["user_agent"])

    def get(self, uri, disable_proxy=False, stream=False):
        """Return Requests response to GET request."""
        proxy = self.proxy if not disable_proxy else False

        return requests.get(
            uri,
            headers=self.headers,
            allow_redirects=True,
            cookies={},
            stream=stream,
            proxies=proxy
        )

    def get_content(self, uri, disable_proxy=False):
        """Return content from URI if Response status is good."""
        response = self.get(uri=uri, disable_proxy=disable_proxy)

        if response.status_code in _PERMITTED_STATUS_CODES:
            self.response_headers = response.headers
            return response.content
        else:
            raise requests.exceptions.HTTPError(
                "HTTP response did not have a permitted status code."
            )

    def get_iter_content(self, uri, disable_proxy=False):
        """Return iterable content from URI if Response status is good."""
        response = self.get(uri=uri, disable_proxy=disable_proxy, stream=True)

        if response.status_code in _PERMITTED_STATUS_CODES:
            self.response_headers = response.headers
            return response.iter_content()
        else:
            raise requests.exceptions.HTTPError(
                "HTTP response did not have a permitted status code."
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Command-line interface to clear Redis."""

import argparse
from parker.workers import killer

parser = argparse.ArgumentParser(
    description=(
        'Clear up the Redis stuff for the specified site.'
    )
)
parser.add_argument(
    'site',
    type=str,
    help='the name of the site'
)
args = parser.parse_args()

killer(args.site)



"""
Parker
-----

Parker is a Python-based web spider for collecting specific data across a set
of configured sites.

"""

from setuptools import setup, find_packages
from pip.req import parse_requirements

version_file = open('VERSION')
version = version_file.read().strip()

install_reqs = parse_requirements('requirements.txt')
reqs = [str(ir.req) for ir in install_reqs]

setup(
    name='Parker',
    version=version,
    url='http://github.com/nefarioustim/parker/',
    license='GPL v3',
    author='Tim Huegdon',
    author_email='tim@nefariousdesigns.co.uk',
    description='A web spider for collecting specific data across a set of '
                'configured sites',
    long_description=__doc__,
    install_requires=reqs,
    packages=['parker']
)



# -*- coding: utf-8 -*-
"""Worker methods for Parker."""

import os.path
from consumemodel import get_instance as get_consume
from crawlmodel import get_instance as get_crawl
from redisset import get_instance as get_redisset
from queues import crawl_q, consume_q
from configloader import load_site_config
from fileops import IMG_DIR, DATA_DIR

_get_instance = {
    'consume': get_consume,
    'crawl': get_crawl
}


def consumer(site, uri):
    """Consume URI using site config."""
    model = _get_model('consume', site, uri)
    model.save_media_to_file(IMG_DIR)
    model.save_to_file(
        os.path.join(
            DATA_DIR,
            site + '.data'
        )
    )


def crawler(site, uri=None):
    """Crawl URI using site config."""
    visited_set, consume_set = _get_site_sets(site)
    model = _get_model('crawl', site, uri)

    if not visited_set.has(model.hash):
        visited_set.add(model.hash)

        if (
            model.is_consume_page
            and not consume_set.has(model.hash)
        ):
            consume_set.add(model.hash)
            consume_q.enqueue(
                consumer,
                site,
                model.uri
            )
        else:
            _enqueue_crawl_uris(site, model.uris_to_crawl)


def killer(site):
    """Kill queues and Redis sets."""
    visited_set, consume_set = _get_site_sets(site)

    crawl_q.empty()
    consume_q.empty()

    visited_set.destroy()
    consume_set.destroy()


def _get_site_sets(site):
    return (
        get_redisset(
            "%s:%s" % (site, 'visited')
        ),
        get_redisset(
            "%s:%s" % (site, 'consume')
        )
    )


def _get_model(model_name, site, uri):
    config = load_site_config(site)
    model = _get_instance[model_name](
        uri if uri is not None else config.get('uri_start_crawl', None)
    )
    model.load_from_config(config)
    return model


def _enqueue_crawl_uris(site, uris):
    for crawl_uri in uris:
        crawl_q.enqueue(
            crawler,
            site,
            crawl_uri
        )
